#-----------------------------------------------------------------------------
# Project    : Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File       : sim.tcl
# Authors    : Mickael Fiorentino <mickael.fiorentino@polymtl.ca>
# Lab        : GRM - Polytechnique Montréal
# Date       : <2019-07-24 Wed>
#-----------------------------------------------------------------------------
# Description: Script pour les simulations du compteur BCD avec Modelsim
# HOW-TO     : source setup.csh && cd simulation/
#            : (CLI) > vsim -c -do ${SCRIPTS_DIR}/sim.tcl
#            : (GUI) > vsim -do ${SCRIPTS_DIR}/sim.tcl
#-----------------------------------------------------------------------------

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_BEH  1;  # Simulation du modèle comportemental
set DO_SYN  0;  # Smulation de la netlist post-synthèse
set DO_PNR  0;  # Simulation de la netlist post-implémentation
set DO_RUN  0;  # 0=Compilation; 1=Compilation + Simulation
set DO_WAVE 0;  # Chargement des chronogrammes
set DO_VCD  0;  # Enregistrement de l'activité dans un fichier vcd

#-----------------------------------------------------------------------------
# DEFAULTS
#-----------------------------------------------------------------------------
set top    "compteur"
set tb     "${top}_tb"
set dut    "dut"
set srcD   "$::env(PROJECT_HOME)/sources"
set netLib "$::env(SIMLIB_VER)"

if { $DO_BEH } {
    set work "$::env(SIM_BEH_DIR)/work"    
    set wave "$::env(SIM_BEH_DIR)/${top}.beh.wave.do"
    set vcd  "$::env(SIM_BEH_DIR)/${top}.beh.vcd"    
} elseif { $DO_SYN } {    
    set work "$::env(SIM_SYN_DIR)/work"
    set wave "$::env(SIM_SYN_DIR)/${top}.syn.wave.do"
    set vcd  "$::env(SIM_SYN_DIR)/${top}.syn.vcd"
    set net  "$::env(SYN_NET_DIR)/${top}.syn.v"    
    set sdf  "$::env(SYN_NET_DIR)/${top}.syn.sdf"    
} elseif { $DO_PNR } {    
    set work "$::env(SIM_PNR_DIR)/work"
    set wave "$::env(SIM_PNR_DIR)/${top}.pnr.wave.do"
    set vcd  "$::env(SIM_PNR_DIR)/${top}.pnr.vcd"    
    set net  "$::env(PNR_NET_DIR)/${top}.pnr.v"
    set sdf  "$::env(PNR_NET_DIR)/${top}.pnr.sdf"
} else {
    error "Au moins une des trois options doit être spécifiée"
}

#-----------------------------------------------------------------------------
# WORK LIB
#-----------------------------------------------------------------------------
if { [file exist $work] } {	
	vdel -all -lib $work
}
vlib $work

#-----------------------------------------------------------------------------
# COMPILATION/SIMULATION
#-----------------------------------------------------------------------------
if { $DO_BEH } {

    vcom -2008 -work $work ${srcD}/${top}.vhd
    vcom -2008 -work $work ${srcD}/${tb}.vhd
    vsim -t ps ${work}.${tb}
    
} elseif { $DO_SYN || $DO_PNR } {

    vlog -work $work $net
    vcom -2008 -work $work ${srcD}/${tb}.vhd
    vsim -L $netLib -t ps -sdfmax ${dut}=${sdf} ${work}.${tb} 
}

#-----------------------------------------------------------------------------
# RUN
#-----------------------------------------------------------------------------
if { $DO_RUN } {
    
    if { $DO_WAVE } { do $wave }
    if { $DO_VCD }  { vcd file $vcd; vcd add -r /${tb}/${dut}/* }
    run -all
    if { $DO_VCD }  { vcd flush }
}
