-------------------------------------------------------------------------------
-- Project    : Tutoriels - Conception de circuits intégrés numériques
-------------------------------------------------------------------------------
-- File       : compteur_tb.vhd
-- Author     : Mickael Fiorentino <mickael.fiorentino@polymtl.ca>
-- Lab        : grm@polymtl
-- Date       : 2019-07-24
-------------------------------------------------------------------------------
-- Description: Banc d'essai du compteur BCD 
-------------------------------------------------------------------------------
library ieee; 
use     ieee.std_logic_1164.all;

library std;
use     std.textio.all;                                                      
use     std.env.all;

library work;
use     work.all;

entity compteur_tb is 
end compteur_tb;

architecture tb of compteur_tb is

  component compteur is
    port (
      i_clk  : in  std_logic;
      i_rstn : in  std_logic;
      i_en   : in  std_logic;
      o_cnt  : out std_logic_vector(3 downto 0));
  end component compteur;
  
  signal clk   : std_logic := '0';
  signal rst_n : std_logic := '0';
  signal en    : std_logic := '0';
  signal cnt   : std_logic_vector(3 downto 0);

  constant PERIOD   : time := 1 ns;
  constant TB_LOOP  : positive := 2;
  constant EXPECTED : std_logic_vector(3 downto 0) := "0011";
  
begin
  

  -- Clock
  clk <= not clk after PERIOD / 2;

  -- DUT
  dut: compteur
    port map (
      i_clk  => clk,
      i_rstn => rst_n,
      i_en   => en,
      o_cnt  => cnt);

  -- Main TB process
  p_main : process
  begin
    report "SIMULATION STARTS";

    for i in 0 to TB_LOOP-1 loop
      en    <= '0';
      rst_n <= '0';
      wait for 2.3 * PERIOD;
      rst_n <= '1';
      en    <= '1';
      wait for 15 * PERIOD;
      en    <= '0';

      -- Assertion
      wait for 1*PERIOD;
      assert cnt = EXPECTED
        report " cnt = " & to_hstring(cnt) & ", should be = " & to_hstring(EXPECTED)
        severity WARNING;
      wait for 1*PERIOD;
      
    end loop;
    
    report "SIMULATION ENDS";
    stop;      
  end process p_main;
  
end architecture tb;
